from .wbits import Wonderbits



class Observer(Wonderbits):
    def __init__(self, index = 1):
        Wonderbits.__init__(self)
        self.index = index

    
    def register_temperature(self, cb):
        self._register_event('observer{}'.format(self.index), 'temperature', cb)
    
    def register_humidity(self, cb):
        self._register_event('observer{}'.format(self.index), 'humidity', cb)
    
    def register_light(self, cb):
        self._register_event('observer{}'.format(self.index), 'light', cb)
    
    def register_volume(self, cb):
        self._register_event('observer{}'.format(self.index), 'volume', cb)
    
    def get_temperature(self):
        """该函数用于获取模块检测的温度值，单位,°C """
        command = 'observer{}.get_temperature()'.format(self.index)
        self._get_command(command)
        return self._result
    
    def get_humidity(self):
        """该函数用于获取模块检测的湿度值，这里测量的湿度为相对湿度单位,%RH """
        command = 'observer{}.get_humidity()'.format(self.index)
        self._get_command(command)
        return self._result
    
    def get_light(self):
        """该函数用于获取模块检测的亮度值，这里的亮度值代表一种量级无单位，值越大代表亮度越强。,用手遮挡住传感器的无光环境监测值为0，使用手机闪光灯发光时对准检测亮度传感器值为100。 """
        command = 'observer{}.get_light()'.format(self.index)
        self._get_command(command)
        return self._result
    
    def get_volume(self):
        """该函数用于获取模块检测的声音强度值，这里的亮度值代表一种量级无单位，值越大代表声音强度越强。,安静为0，声源需要靠近传感器效果会更好。 """
        command = 'observer{}.get_volume()'.format(self.index)
        self._get_command(command)
        return self._result
    



import time
import json
from .wbSerial import WBSerial
import threading
import os

class Wonderbits(object):
    '''
    单例类
    该类是所有豌豆拼模块的父类，负责和串口类WBSerial交互
    接受并响应豌豆拼模块发送的指令
    '''

    # 该类初始化标志
    _init_flag = False
    # 供获取类的函数使用
    _result = None
    # 供设置类完成返回标志
    _command = None
    # 存储事件注册的key，value
    _event_call_back_dict = {}
    # link-socket通信的串口
    _wbSerial = None

    def __init__(self):
        if Wonderbits._init_flag:
            return
        print('wonderbits初始化！')
        try:
            Wonderbits._wbSerial = WBSerial()
            Wonderbits._wbSerial.event_data_cb = self._event_data_from_local_py_serial
            Wonderbits._wbSerial.reporter_data_cb = self._reporter_data_from_local_py_serial
            Wonderbits._wbSerial.command_finish_cb = self._command_data_from_local_py_serial
            Wonderbits._init_flag = True
        except Exception as e:
            print('wonderbits初始化失败!', e)
                    
    def _event_data_from_local_py_serial(self, data):
        '''
        本地串口通信的事件通知
        '''
        self._parse_event_data_and_notify(data)
    
    def _reporter_data_from_local_py_serial(self, value):
        '''
        本地串口通信的返回类函数命令完成的通知
        '''
        Wonderbits._result = self._format_str_to_bool_int_float(value)

    def _command_data_from_local_py_serial(self, key):
        '''
        本地串口通信的设置类命令完成的通知
        '''
        Wonderbits._command = key.strip()

    def _parse_event_data_and_notify(self, data):
        '''
        解析数据，并且通知调用者 
        '''
        try:
            obj = json.loads(data)
            if obj['type'] == 'event':
                key = '{}.{}'.format(obj['module'], obj['source'])
                if Wonderbits._event_call_back_dict[key]:
                    return_value = self._format_str_to_bool_int_float(obj['value'])
                    # { "module": obj['module'], "source": obj['source'], "value":obj['value'] }
                    Wonderbits._event_call_back_dict[key](return_value)
        except:
            pass

    def _register_event(self, module_name, source, cb):
        '''
        事件注册类
        '''
        cmd = '{}.register.{}()'.format(module_name, source)
        self._wbSerial.write_command(cmd)
        Wonderbits._event_call_back_dict['{}.{}'.format(module_name, source)] = cb

    def _set_command(self, command):
        '''
        设置类
        '''
        self._wbSerial.write_command(command)
        Wonderbits._command = None
        self._set_command_time_out(3, command)

    def _get_command(self, command):
        '''
        获取类
        '''
        self._wbSerial.write_command(command)
        Wonderbits._result = None
        self._set_time_out(3)

    def _set_command_time_out(self, loop_forever_time=0.1, break_value = None):
        '''
        供设置类命令 使用
        给sdk中设置类函数使用---自定义sleep(采用超时机制)
        '''
        try:
            time_interval = 0.001
            count = loop_forever_time // time_interval
            while count > 0:
                if Wonderbits._command == break_value or Wonderbits._command == 'error':
                    break
                time.sleep(time_interval)
                count = count - 1
        except KeyboardInterrupt as e:
            print('exit-wb')
            os._exit(0)
        
    def _set_time_out(self, loop_forever_time=0.1, break_value=None):
        '''
        供获取类命令 使用
        给sdk中带返回值函数使用---自定义sleep(采用超时机制)
        '''
        try:
            time_interval = 0.001
            count = loop_forever_time // time_interval
            while count > 0:
                if Wonderbits._result != break_value:
                    break
                time.sleep(time_interval)
                count = count - 1
        except KeyboardInterrupt as e:
            print('exit-wb')
            os._exit(0)

    def _format_str_to_bool_int_float(self, data):
        '''
        工具函数：将字符串转成bool int float
        '''
        return_value = data
        try:
            if data.startswith('\'') and data.endswith('\''):
                return_value = data
            elif data == 'True':
                return_value = True
            elif data == 'False':
                return_value = False
            elif data.find(".") != -1:
                return_value = float(return_value)
            elif data.find(".") == -1:
                return_value = int(return_value)
            return return_value
        except Exception as e:
            pass



from .wbits import Wonderbits



class RfCommunication(Wonderbits):
    def __init__(self, index = 1):
        Wonderbits.__init__(self)
        self.index = index

    
    def register_button(self, cb):
        self._register_event('rfCommunication{}'.format(self.index), 'button', cb)
    
    def register_msg_received(self, cb):
        self._register_event('rfCommunication{}'.format(self.index), 'msg_received', cb)
    
    def get_msg(self):
        """使用该函数可得到最近一次通信收到的内容，如果在程序开始后或使用clear_msg函数后没有发生过通信将返回None """
        command = 'rfCommunication{}.get_msg()'.format(self.index)
        self._get_command(command)
        return self._result
    
    def clear_msg(self):
        """清除最新的通信内容，在再次接收到新的通信内容之前调用get_msg只会返回None,调用此函数并不会影响get_unread_msg_count和read的使用 """
        command = 'rfCommunication{}.clear_msg()'.format(self.index)
        self._set_command(command)
    
    def get_unread_msg_count(self):
        """该函数用于获取通信存储队列中未读内容的个数，最多存储32个未读内容 """
        command = 'rfCommunication{}.get_unread_msg_count()'.format(self.index)
        self._get_command(command)
        return self._result
    
    def read(self):
        """该函数用于获取通信存储队列中未读内容，读取后会删除这个数据 """
        command = 'rfCommunication{}.read()'.format(self.index)
        self._get_command(command)
        return self._result
    
    def send(self, number):
        """发送数据。调用此函数后，与本模块通信名字相同的模块将会受到发送的内容 """
        command = 'rfCommunication{}.send({})'.format(self.index,number)
        self._set_command(command)
    
    def init(self, name):
        """设置模块通信名字。只有通信名字相同的模块之间才可以互相通信，不想互相通信的模块需要设置不同的通信名字 """
        name = name.replace('"', '\\"')
        command = 'rfCommunication{}.init({})'.format(self.index,name)
        self._set_command(command)
    
    def is_button_pressed(self):
        """该函数用于获取按键是否被按下 """
        command = 'rfCommunication{}.is_button_pressed()'.format(self.index)
        self._get_command(command)
        return self._result
    


from .wbits import Wonderbits



class Ultrasonic(Wonderbits):
    def __init__(self, index = 1):
        Wonderbits.__init__(self)
        self.index = index

    
    def register_distance(self, cb):
        self._register_event('ultrasonic{}'.format(self.index), 'distance', cb)
    
    def get_distance(self):
        """该函数用于获取超声波检测的距离值，单位：cm """
        command = 'ultrasonic{}.get_distance()'.format(self.index)
        self._get_command(command)
        return self._result
    


import serial
import serial.tools.list_ports
import time
import threading
import os

class WBSerial(object):
    '''
    单例类
    主要负责串口通信： 连接，收发串口数据，处理串口数据
    '''

    # 初始化标志
    _init_flag = False
    # 是否开启控制台输出串口数据日志
    _is_show_console = False

    def __init__(self):
        if WBSerial._init_flag:
            return
        WBSerial._init_flag = True
        # 初始化一些必要参数，出错时会重置这些参数
        self._initProperty()
        # 连接成功的回调函数
        self.connected_cb = None
        
        # 事件数据的回调函数
        self.event_data_cb = None #ddd
        # 带返回值数据命令的回调
        self.reporter_data_cb = None
        
        # 设置类命令的回调函数
        self._command_finish = None
        self.command_finish_cb = None

        # 开启一个子线程进行轮询串口
        # 若发现有我们的板子，就会尝试连接下
        # 如果板子有多块，第一块连不上的情况下会自动连接第二块，依次类推，直到连上我们的板子位置
        threading.Thread(target=self._create_try_connect_py_serial_thread, args=('_create_try_connect_py_serial_thread',)).start()
    
    def _initProperty(self):
        '''
        初始化该类的一些属性
        '''
        self._ser = None
        self._canUsedPort = []
        self._initIndex = 0
        self._canUseCount = 0
        self._canSend = False
    
    def write_command(self, command):
        '''
        发送命令给串口
        '''
        try:
            cmd = '{}\r\n'.format(command).encode('gbk')
            while not self._canSend:
                time.sleep(.001)
            self._canSend and self._ser.write(cmd)
        except KeyboardInterrupt as e:
            print('exit-wb')
            os._exit(0)

    def _create_listen_serial_port_data_event_thread(self, name):
        '''
        监听串口数据(轮询)
        '''
        _buffer = ''
        try:
            while True:
                oneByte = self._ser.read(1)
                _buffer += oneByte.decode("gbk")
                if oneByte == b"\n":
                    if _buffer.find('Traceback') != -1:
                        self.reporter_data_cb and threading.Thread(target=self._create_return_serial_data_thread, args=('_create_return_serial_data_thread', "-1")).start()
                        _buffer = ''
                        continue
                    self._handle_serial_data(_buffer)
                    self._command_finish = _buffer
                    _buffer = ''
                if oneByte == b">" and _buffer[-3:] == '>>>':
                    self._handle_serial_data(_buffer)
                    _buffer = ''
                    if self._command_finish and self.command_finish_cb:
                        # 设置类命令 完成时通知
                        threading.Thread(target=self._create_return_command_finished_thread, args=('_create_return_command_finished_thread',self._command_finish)).start()
                        self._command_finish = None
                
        except KeyboardInterrupt as e:
            print('exit-wb')
            os._exit(0)

        except OSError as e:
            print('设备未配置')
            self._initProperty()

        except Exception as e:
            print("串口异常：{}".format(e))
            self._initProperty()

    def _handle_serial_data(self, r_str = ''):
        '''
        处理串口数据
        '''
        if not r_str:
            return
        if WBSerial._is_show_console:
            print(r_str, end="")
        receive_str = r_str.strip()
        if receive_str.find('Type "help()" for more') != -1:
            self._canSend = True
            self.connected_cb and self.connected_cb()
        elif receive_str.startswith("{") and receive_str.endswith('}') and self.event_data_cb:
            # 事件触发返回数据
            threading.Thread(target=self._create_return_event_data_thread, args=('_create_return_event_data_thread',receive_str)).start()
        elif not receive_str.endswith(")") and receive_str.find('>>>') == -1 and self.reporter_data_cb:
            # 获取类返回值
            threading.Thread(target=self._create_return_serial_data_thread, args=('_create_return_serial_data_thread', receive_str)).start()

    def _create_return_event_data_thread(self, name, result):
        '''
        事件类通知回调
        '''
        self.event_data_cb(result)

    def _create_return_serial_data_thread(self, name, value):
        '''
        获取类执行完毕回调
        '''
        self.reporter_data_cb(value)

    def _create_return_command_finished_thread(self, name, key):
        '''
        设置类执行完毕回调
        '''
        self.command_finish_cb(key)

    def _initConnect(self):
        '''
        初始化连接本地串口，获取串口信息
        '''
        if self._ser != None:
            return
        self._initProperty()
        port_list = list(serial.tools.list_ports.comports())
        for i in range(len(port_list)):
            port = port_list[i]
            if port.pid == 29987 or port.pid == 60000:
                self._canUsedPort.append(port)
                # print(port.device, port.pid, port.vid, port.hwid)
                self._canUseCount = len(self._canUsedPort)

    def _connect_serial(self, index=0):
        '''
        根据指定端口，连接本地串口
        '''
        try:
            portx = self._canUsedPort[index].device
            bps = 115200
            timex = 1
            self._ser = serial.Serial(portx, bps, timeout=timex)
            cmd = '{}\r\n'.format("reset()").encode('gbk')
            self._ser.write(cmd)
        
            # 监听串口返回的数据
            threading.Thread(target=self._create_listen_serial_port_data_event_thread, args=('_create_listen_serial_port_data_event_thread',)).start()
            
        except serial.serialutil.SerialException as e:
            print("串口异常{}".format(e))
            if self._initIndex < self._canUseCount - 1:
                self._connect_serial(self._initIndex)
            else:
                self._initProperty()

        except Exception as e:
            print("通用异常：{}".format(e))
            self._initProperty()

    def _create_try_connect_py_serial_thread(self, thread_name):
        '''
        监听串口变化，初始化或者连接
        '''
        try:
            while True:
                self._initConnect()
                if not self._canUseCount:
                    print('无可用串口')
                else:
                    if self._ser == None and self._canUseCount != 0:
                        self._connect_serial()
                time.sleep(.5)
        except KeyboardInterrupt as e:
            print('exit-wb')
            os._exit(0)


from .wbits import Wonderbits



class Led(Wonderbits):
    def __init__(self, index = 1):
        Wonderbits.__init__(self)
        self.index = index

    
    def set_rgb(self, r, g, b):
        """设置彩灯颜色 """
        command = 'led{}.set_rgb({},{},{})'.format(self.index,r,g,b)
        self._set_command(command)
    
    def fade_to_rgb(self, r, g, b, total, step = 50, key = False):
        """设置彩灯由当前颜色渐变到目标颜色 """
        command = 'led{}.fade_to_rgb({},{},{},{},{},{})'.format(self.index,r,g,b,total,step,key)
        self._set_command(command)
    


from .wbTool import wb_tool

from .Control import Control
from .Display import Display
from .Led import Led
from .RfCommunication import RfCommunication
from .RfTelecontroller import RfTelecontroller
from .Driver import Driver
from .Hall import Hall
from .LightBelt import LightBelt
from .Observer import Observer
from .Pulse import Pulse
from .Signal import Signal
from .Ultrasonic import Ultrasonic
from .Acceleration import Acceleration


from .wbits import Wonderbits



class LightBelt(Wonderbits):
    def __init__(self, index = 1):
        Wonderbits.__init__(self)
        self.index = index

    
    def set_leds_rgb(self, start, end, r, g, b):
        """设置灯带上某一段灯的颜色 """
        command = 'lightBelt{}.set_leds_rgb({},{},{},{},{})'.format(self.index,start,end,r,g,b)
        self._set_command(command)
    
    def set_single_led_rgb(self, num, r, g, b):
        """设置灯带上某个灯的颜色 """
        command = 'lightBelt{}.set_single_led_rgb({},{},{},{})'.format(self.index,num,r,g,b)
        self._set_command(command)
    


from .wbits import Wonderbits



class Hall(Wonderbits):
    def __init__(self, index = 1):
        Wonderbits.__init__(self)
        self.index = index

    
    def register_magnetic(self, cb):
        self._register_event('hall{}'.format(self.index), 'magnetic', cb)
    
    def get_magnetic(self):
        """该函数用于获取霍尔检测的磁场强度值 """
        command = 'hall{}.get_magnetic()'.format(self.index)
        self._get_command(command)
        return self._result
    
    def calibrate(self):
        """校准霍尔传感器零点,使用该函数时，霍尔模块指示灯会在校准执行过程中变为黄色，校准完成后回复原有颜色。,校准过程中保证没有磁性物体靠近模块，否则会导致校准后的零点不准确。 """
        command = 'hall{}.calibrate()'.format(self.index)
        self._set_command(command)
    


from .wbits import Wonderbits



class Display(Wonderbits):
    '''
    显示模块
    '''
    def __init__(self, index = 1):
        Wonderbits.__init__(self)
        self.index = index

    def register_button(self, cb):
        self._register_event('display{}'.format(self.index), 'button', cb)
    
    def print(self, row, column, text, size = 2):
        """固定位置显示 """
        text = str(text).replace('"', '\\"')
        command = 'display{}.print({},{},"{}",{})'.format(self.index,row,column,text,size)
        self._set_command(command)
    
    def draw_dot(self, x, y, page, save = 0, color = 1):
        """画点 """
        command = 'display{}.draw_dot({},{},{},{},{})'.format(self.index,x,y,page,save,color)
        self._set_command(command)
    
    def draw_line(self, head_x, head_y, tail_x, tail_y, page, save = 0, color = 1):
        """画线 """
        command = 'display{}.draw_line({},{},{},{},{},{},{})'.format(self.index,head_x,head_y,tail_x,tail_y,page,save,color)
        self._set_command(command)
    
    def turn_to_page(self, page):
        """转到某页 """
        command = 'display{}.turn_to_page({})'.format(self.index,page)
        self._set_command(command)
    
    def clear_page(self, page):
        """清除某页 """
        command = 'display{}.clear_page({})'.format(self.index,page)
        self._set_command(command)
    
    def clear_all_pages(self):
        """清除全部8页屏幕的内容 """
        command = 'display{}.clear_all_pages()'.format(self.index)
        self._set_command(command)
    
    def get_button_state(self):
        """该函数用于获取翻页按钮状态 """
        command = 'display{}.get_button_state()'.format(self.index)
        self._get_command(command)
        return self._result
    
    def disable_page_turning(self):
        """禁止翻页按键功能,在开启翻页按键功能的情况下使用该函数可以禁止翻页按键功能，禁止翻页按键功能后将不能通过翻页按键来切换不同页码的显示内容，只能使用turn_to_page函数来切换页码,系统默认开启翻页按键功能 """
        command = 'display{}.disable_page_turning()'.format(self.index)
        self._set_command(command)
    
    def enable_page_turning(self):
        """开启翻页按键功能,在禁止翻页按键功能的情况下使用该函数可以开启翻页按键功能,系统默认开启翻页按键功能 """
        command = 'display{}.enable_page_turning()'.format(self.index)
        self._set_command(command)
    
    def set_direction_reverse(self):
        """设置显示方向为翻转显示方向，使用该函数后显示内容将会进行180°翻转 """
        command = 'display{}.set_direction_reverse()'.format(self.index)
        self._set_command(command)
    
    def set_direction_regular(self):
        """设置显示方向为系统默认显示方向 """
        command = 'display{}.set_direction_regular()'.format(self.index)
        self._set_command(command)
    
    def hide_scrollbar(self):
        """隐藏页码滚动指示条，使用该函数后将不会再显示内容界面看到页码滚动指示条,系统默认显示页码滚动指示条 """
        command = 'display{}.hide_scrollbar()'.format(self.index)
        self._set_command(command)
    
    def show_scrollbar(self):
        """显示页码滚动指示条,系统默认显示页码滚动指示条 """
        command = 'display{}.show_scrollbar()'.format(self.index)
        self._set_command(command)
    
    def disable_auto_refresh(self):
        """禁止自动刷新显示功能,在禁止自动刷新显示功能后只能靠手动刷新显示界面实现更新显示内容,系统默认开启自动刷新显示功能 """
        command = 'display{}.disable_auto_refresh()'.format(self.index)
        self._set_command(command)
    
    def enable_auto_refresh(self):
        """开启自动刷新显示功能,在开启自动刷新显示功能后系统将智能识别当前显示内容是否需要更新，如果需要则会更新显示内容,系统默认开启自动刷新显示功能 """
        command = 'display{}.enable_auto_refresh()'.format(self.index)
        self._set_command(command)
    
    def refresh(self):
        """更新显示内容,在禁止自动刷新显示功能后只能靠该函数来实现手动刷新显示界面实现更新显示内容,系统默认开启自动刷新显示功能 """
        command = 'display{}.refresh()'.format(self.index)
        self._set_command(command)
    


from .wbSerial import WBSerial

class wb_tool():
    '''
    豌豆拼工具集合
    '''

    @staticmethod
    def show_console():
        '''
        开启控制台输出
        '''
        WBSerial._is_show_console = True

    @staticmethod
    def hide_console():
        '''
        隐藏控制台输出（默认）
        '''
        WBSerial._is_show_console = False


from .wbits import Wonderbits



class Acceleration(Wonderbits):
    def __init__(self, index = 1):
        Wonderbits.__init__(self)
        self.index = index

    
    def register_x_acceleration(self, cb):
        self._register_event('acceleration{}'.format(self.index), 'x_acceleration', cb)
    
    def register_y_acceleration(self, cb):
        self._register_event('acceleration{}'.format(self.index), 'y_acceleration', cb)
    
    def register_z_acceleration(self, cb):
        self._register_event('acceleration{}'.format(self.index), 'z_acceleration', cb)
    
    def register_acceleration(self, cb):
        self._register_event('acceleration{}'.format(self.index), 'acceleration', cb)
    
    def register_x_angular_velocity(self, cb):
        self._register_event('acceleration{}'.format(self.index), 'x_angular_velocity', cb)
    
    def register_y_angular_velocity(self, cb):
        self._register_event('acceleration{}'.format(self.index), 'y_angular_velocity', cb)
    
    def register_z_angular_velocity(self, cb):
        self._register_event('acceleration{}'.format(self.index), 'z_angular_velocity', cb)
    
    def get_x_acceleration(self):
        """该函数用于获取加速度传感器检测的x轴加速度值，单位,m/s2 """
        command = 'acceleration{}.get_x_acceleration()'.format(self.index)
        self._get_command(command)
        return self._result
    
    def get_y_acceleration(self):
        """该函数用于获取加速度传感器检测的y轴加速度值，单位,m/s2 """
        command = 'acceleration{}.get_y_acceleration()'.format(self.index)
        self._get_command(command)
        return self._result
    
    def get_z_acceleration(self):
        """该函数用于获取加速度传感器检测的z轴加速度值，单位,m/s2 """
        command = 'acceleration{}.get_z_acceleration()'.format(self.index)
        self._get_command(command)
        return self._result
    
    def get_acceleration(self):
        """该函数用于获取加速度传感器检测的x、y、z三轴合加速度值，单位,m/s2 """
        command = 'acceleration{}.get_acceleration()'.format(self.index)
        self._get_command(command)
        return self._result
    
    def get_x_angular_velocity(self):
        """该函数用于获取加速度传感器检测的x轴角速度值，单位,°/s """
        command = 'acceleration{}.get_x_angular_velocity()'.format(self.index)
        self._get_command(command)
        return self._result
    
    def get_y_angular_velocity(self):
        """该函数用于获取加速度传感器检测的y轴角速度值，单位,°/s """
        command = 'acceleration{}.get_y_angular_velocity()'.format(self.index)
        self._get_command(command)
        return self._result
    
    def get_z_angular_velocity(self):
        """该函数用于获取加速度传感器检测的z轴角速度值，单位,°/s """
        command = 'acceleration{}.get_z_angular_velocity()'.format(self.index)
        self._get_command(command)
        return self._result
    
    def calibrate(self):
        """校准加速度传感器零点,使用该函数时，加速度模块指示灯会在校准执行过程中变为黄色，校准完成后回复原有颜色。,校准过程中需要保证加速度模块且保持静止不动，有汉字的一面朝上。 """
        command = 'acceleration{}.calibrate()'.format(self.index)
        self._set_command(command)
    


from .wbits import Wonderbits



class Pulse(Wonderbits):
    def __init__(self, index = 1):
        Wonderbits.__init__(self)
        self.index = index

    
    def register_heart_rate(self, cb):
        self._register_event('pulse{}'.format(self.index), 'heart_rate', cb)
    
    def register_heart_wave_received(self, cb):
        self._register_event('pulse{}'.format(self.index), 'heart_wave_received', cb)
    
    def get_heart_rate(self):
        """该函数用于获取模块检测的脉搏，此处的脉搏值表示一分钟脉搏跳动的次数,测量时要求找到模块有汉字的一面，然后将手指轻轻的贴在此面，需要耐心等待一会则会测量出脉搏 """
        command = 'pulse{}.get_heart_rate()'.format(self.index)
        self._get_command(command)
        return self._result
    
    def get_unread_wave_count(self):
        """该函数用于获取脉搏波形队列中未读内容的个数，最多存储10个未读内容 """
        command = 'pulse{}.get_unread_wave_count()'.format(self.index)
        self._get_command(command)
        return self._result
    
    def get_heart_wave(self):
        """该函数用于获取脉搏波形队列中的未读波形值，读取后会删除这个数据,如果没有未读的数据返回上一次的返回值 """
        command = 'pulse{}.get_heart_wave()'.format(self.index)
        self._get_command(command)
        return self._result
    


from .wbits import Wonderbits



class Control(Wonderbits):
    def __init__(self, index=1):
        Wonderbits.__init__(self)
        self.index = index
        self.event = Control._Event(self)

    class _Event():
        def __init__(self, this):
            self.this = this

        def sw1_pressed(self):
            def wrapper(cb):
                self.this._register_event(
                    'control{}'.format(self.this.index), 'sw1', cb)
            return wrapper

        def sw2_pressed(self):
            def wrapper(cb):
                self.this._register_event(
                    'control{}'.format(self.this.index), 'sw2', cb)
            return wrapper

    
    def register_sw1(self, cb):
        self._register_event('control{}'.format(self.index), 'sw1', cb)
    
    def register_sw2(self, cb):
        self._register_event('control{}'.format(self.index), 'sw2', cb)
    
    def register_sw3(self, cb):
        self._register_event('control{}'.format(self.index), 'sw3', cb)
    
    def register_sw4(self, cb):
        self._register_event('control{}'.format(self.index), 'sw4', cb)
    
    def register_m1(self, cb):
        self._register_event('control{}'.format(self.index), 'm1', cb)
    
    def register_m2(self, cb):
        self._register_event('control{}'.format(self.index), 'm2', cb)
    
    def register_m1_value(self, cb):
        self._register_event('control{}'.format(self.index), 'm1_value', cb)
    
    def register_m2_value(self, cb):
        self._register_event('control{}'.format(self.index), 'm2_value', cb)
    
    def is_sw1_pressed(self):
        """该函数用于判断SW1是否被按下 """
        command = 'control{}.is_sw1_pressed()'.format(self.index)
        self._get_command(command)
        return self._result
    
    def is_sw2_pressed(self):
        """该函数用于判断SW2是否被按下 """
        command = 'control{}.is_sw2_pressed()'.format(self.index)
        self._get_command(command)
        return self._result
    
    def is_sw3_at_1(self):
        """该函数用于判断SW3的是否在1这侧 """
        command = 'control{}.is_sw3_at_1()'.format(self.index)
        self._get_command(command)
        return self._result
    
    def get_sw4(self):
        """该函数用于判断获取SW4的位置 """
        command = 'control{}.get_sw4()'.format(self.index)
        self._get_command(command)
        return self._result
    
    def is_m1_connected(self):
        """该函数用于判断获取M1与COM是否导通，导通的判断是根据M1与COM之间的电阻率是否低于阈值，低于阈值判断为导通，高于阈值判断为不导通 """
        command = 'control{}.is_m1_connected()'.format(self.index)
        self._get_command(command)
        return self._result
    
    def is_m2_connected(self):
        """该函数用于判断获取M2与COM是否导通，导通的判断是根据M2与COM之间的电阻率是否低于阈值，低于阈值判断为导通，高于阈值判断为不导通 """
        command = 'control{}.is_m2_connected()'.format(self.index)
        self._get_command(command)
        return self._result
    
    def set_m1_m2_sensitivity(self, limit):
        """设置触摸灵敏度,通过设置灵敏度改变M1，M2的触发阈值,当get_m1_value或get_m2_value小于阈值时则认为M1或M2与COM导通 """
        command = 'control{}.set_m1_m2_sensitivity({})'.format(self.index,limit)
        self._set_command(command)
    
    def get_m1_value(self):
        """该函数用于获取M1的电阻率 """
        command = 'control{}.get_m1_value()'.format(self.index)
        self._get_command(command)
        return self._result
    
    def get_m2_value(self):
        """该函数用于获取M2的电阻率 """
        command = 'control{}.get_m2_value()'.format(self.index)
        self._get_command(command)
        return self._result
    


from .wbits import Wonderbits



class Driver(Wonderbits):
    def __init__(self, index = 1):
        Wonderbits.__init__(self)
        self.index = index

    
    def set_motor_a(self, speed, time = 10):
        """设置电机A转动 """
        command = 'driver{}.set_motor_a({},{})'.format(self.index,speed,time)
        self._set_command(command)
    
    def stop_motor_a(self):
        """设置电机A停止转动 """
        command = 'driver{}.stop_motor_a()'.format(self.index)
        self._set_command(command)
    
    def set_motor_b(self, speed, time = 10):
        """设置电机B转动 """
        command = 'driver{}.set_motor_b({},{})'.format(self.index,speed,time)
        self._set_command(command)
    
    def stop_motor_b(self):
        """设置电机B停止转动 """
        command = 'driver{}.stop_motor_b()'.format(self.index)
        self._set_command(command)
    
    def set_servo1(self, angle):
        """设置舵机1转动到指定角度,使用此函数后舵机1将拥有维持角度的扭矩，施加外力改变舵机1的角度会很困难 """
        command = 'driver{}.set_servo1({})'.format(self.index,angle)
        self._set_command(command)
    
    def shut_servo1(self):
        """关闭舵机1,使用此函数后舵机1将失去维持角度的扭矩，施加外力可以轻松改变舵机1的角度 """
        command = 'driver{}.shut_servo1()'.format(self.index)
        self._set_command(command)
    
    def set_servo2(self, angle):
        """设置舵机2转动到指定角度,使用此函数后舵机2将拥有维持角度的扭矩，施加外力改变舵机2的角度会很困难 """
        command = 'driver{}.set_servo2({})'.format(self.index,angle)
        self._set_command(command)
    
    def shut_servo2(self):
        """关闭舵机2,使用此函数后舵机2将失去维持角度的扭矩，施加外力可以轻松改变舵机2的角度 """
        command = 'driver{}.shut_servo2()'.format(self.index)
        self._set_command(command)
    


from .wbits import Wonderbits



class Signal(Wonderbits):
    def __init__(self, index = 1):
        Wonderbits.__init__(self)
        self.index = index

    
    def set_rgb(self, r, g, b):
        """设置LED灯颜色 """
        command = 'signal{}.set_rgb({},{},{})'.format(self.index,r,g,b)
        self._set_command(command)
    
    def set_buzzer(self, frequency):
        """设置蜂鸣器声音频率，单位,Hz,设置频率为0表示关闭蜂鸣器 """
        command = 'signal{}.set_buzzer({})'.format(self.index,frequency)
        self._set_command(command)
    
    def set_vibration(self, strength):
        """设置震动马达的震动幅度,这里的振动幅度没有单位，值越大表示震动幅度越大，参数为0则停止震动 """
        command = 'signal{}.set_vibration({})'.format(self.index,strength)
        self._set_command(command)
    
    def play_a_note(self, frequency, time):
        """设置蜂鸣器以一个固定频率发声并保持一段时间后关闭蜂鸣器 """
        command = 'signal{}.play_a_note({},{})'.format(self.index,frequency,time)
        self._set_command(command)
    